package com.smartframework.core.websocket;

package ..deTemp.1731158474601;

import com.smartframework.core.websocket.WebSocketMessage;
import com.smartframework.core.websocket.WebSocketService;
import com.smartframework.core.websocket.WebSocketThread;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import org.apache.log4j.Logger;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

public class WebSocketServiceManager {
  private static List<Class<?>> serviceClasses = new LinkedList<>();
  
  private static Logger logger = Logger.getLogger(com.smartframework.core.websocket.WebSocketServiceManager.class);
  
  private static Map<String, String> webSocketSessionHttpSession = new ConcurrentHashMap<>();
  
  private static WebSocketThread workThread = null;
  
  public static final String WEB_SOCKET_HTTP_SESSION_ID = "WEB_SOCKET_HTTP_SESSION_ID";
  
  static final String SMART_MQ_URI_SUFFIX = "name";
  
  public static void addWebSocketService(Class<?> clazz) {
    serviceClasses.add(clazz);
  }
  
  public static String getHttpSession(WebSocketSession wsSession) {
    if (wsSession == null)
      return null; 
    return webSocketSessionHttpSession.get(wsSession.getId());
  }
  
  public static void start() {
    logger.debug("开始启动WebSocketService");
    List<WebSocketService> webSocketServiceList = new LinkedList<>();
    Map<String, WebSocketService> webSocketServices = new ConcurrentHashMap<>();
    for (Class<?> clazz : serviceClasses) {
      try {
        WebSocketService webSocketService = (WebSocketService)clazz.newInstance();
        WebSocketService _webSocketService = webSocketServices.get(webSocketService.getName());
        if (null != _webSocketService) {
          logger.error("WebSocketService:" + webSocketService.getName() + "已经存在，不能重复创建.");
          continue;
        } 
        logger.debug("创建WebSocketService:" + webSocketService.getName());
        webSocketServices.put(webSocketService.getName(), webSocketService);
        webSocketServiceList.add(webSocketService);
      } catch (InstantiationException|IllegalAccessException e) {
        logger.error("创建WebSocketService对象失败:", e);
      } 
    } 
    workThread = new WebSocketThread(webSocketServices);
    workThread.start();
    for (WebSocketService webSocketService : webSocketServiceList)
      webSocketService.start(); 
    logger.debug("完成启动WebSocketService");
  }
  
  protected static void afterConnectionEstablished(WebSocketSession session) throws Exception {
    logger.debug("WebSocketSession:" + session + "建立连接");
    Object httpSessionId = session.getAttributes().get("WEB_SOCKET_HTTP_SESSION_ID");
    if (httpSessionId != null)
      webSocketSessionHttpSession.put(session.getId(), String.valueOf(httpSessionId)); 
    workThread.registerWebSocketSession(session);
  }
  
  protected static void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
    logger.debug("WebSocketSession:" + session + "错误:" + exception.toString());
    WebSocketThread.WebSocketEntry entry = new WebSocketThread.WebSocketEntry();
    entry.type = WebSocketThread.Type.error;
    entry.payload = exception;
    entry.object = session;
    entry.topic = null;
    workThread.push(entry);
  }
  
  protected static void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
    logger.debug("WebSocketSession:" + session + "关闭:" + closeStatus.toString());
    webSocketSessionHttpSession.remove(session.getId());
    workThread.unregisterWebSocketSession(session);
  }
  
  protected static void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
    String strMessage = (String)message.getPayload();
    logger.debug("接收到WebSocketSession:" + session + " 消息 :" + strMessage);
    WebSocketThread.WebSocketEntry entry = new WebSocketThread.WebSocketEntry();
    entry.type = WebSocketThread.Type.recvive;
    entry.payload = strMessage;
    entry.object = session;
    entry.topic = null;
    workThread.push(entry);
  }
  
  public static void broadCast(String serviceName, String topic, Object message) {
    if (workThread == null)
      return; 
    if (serviceName == null || serviceName.trim().isEmpty())
      return; 
    WebSocketThread.WebSocketEntry entry = new WebSocketThread.WebSocketEntry();
    entry.type = WebSocketThread.Type.broadCast;
    entry.payload = message;
    entry.object = serviceName;
    entry.topic = topic;
    workThread.push(entry);
  }
  
  public static String sessionServiceName(WebSocketSession webSocketSession) {
    String uri = webSocketSession.getUri().toString();
    if (uri.lastIndexOf("name") > 0 && !uri.endsWith("name"))
      return uri.substring(uri.lastIndexOf("name") + "name".length() + 1); 
    logger.info("新建WebSocket链接时请指定Service名称,格式ws://host:port/smartMQ?name=服务名称");
    return "";
  }
  
  public static void sendMessage(WebSocketSession session, String topic, Object message) {
    String str = WebSocketMessage.packMessage(sessionServiceName(session), topic, message);
    if (null == str)
      return; 
    sendMessage(session, str);
  }
  
  protected static void sendMessage(WebSocketSession session, String message) {
    WebSocketThread.WebSocketEntry entry = new WebSocketThread.WebSocketEntry();
    entry.type = WebSocketThread.Type.message;
    entry.payload = message;
    entry.object = session;
    entry.topic = null;
    workThread.push(entry);
  }
  
  protected static void sendMessage(List<WebSocketSession> sessions, String message) {
    WebSocketThread.WebSocketEntry entry = new WebSocketThread.WebSocketEntry();
    entry.type = WebSocketThread.Type.p2pmessage;
    entry.payload = message;
    entry.object = sessions;
    entry.topic = null;
    workThread.push(entry);
  }
}
